@@ -1,3 +1,14 @@
+/*
+ * linc-protocols.c: This file is part of the linc library.
+ *
+ * Authors:
+ *    Elliot Lee     (sopwith@redhat.com)
+ *    Michael Meeks  (michael@ximian.com)
+ *    Mark McLouglin (mark@skynet.ie) & others
+ *
+ * Copyright 2001, Red Hat, Inc., Ximian, Inc.,
+ *                 Sun Microsystems, Inc.
+ */
 #include <config.h>
 
 #include <sys/types.h>
@@ -18,6 +29,8 @@
 #include <linc/linc-protocol.h>
 #include <linc/linc-connection.h>
 
+#include "linc-private.h"
+
 #undef DEBUG
 
 #ifdef HAVE_NETINET_IN_H
@@ -520,8 +533,7 @@ linc_protocol_get_sockinfo_ipv6 (const LINCProtocolInfo  *proto,
 				 gchar                  **portnum)
 {
 	struct sockaddr_in6 *sa_in6 = (struct sockaddr_in6 *)saddr;
-	struct hostent      *host;
-
+	struct hostent      *host = NULL;
 
 	g_assert (proto && saddr && saddr->sa_family == AF_INET6);
 
