@@ -1,21 +1,62 @@
+/*
+ * linc.c: This file is part of the linc library.
+ *
+ * Authors:
+ *    Elliot Lee     (sopwith@redhat.com)
+ *    Michael Meeks  (michael@ximian.com)
+ *    Mark McLouglin (mark@skynet.ie) & others
+ *
+ * Copyright 2001, Red Hat, Inc., Ximian, Inc.,
+ *                 Sun Microsystems, Inc.
+ */
 #include <signal.h>
 #include "linc-private.h"
 
-GMainLoop    *linc_loop = NULL;
-GMainContext *linc_context = NULL;
+#undef WATCH_DEBUG
+
+static gboolean linc_threaded = FALSE;
+static gboolean linc_mutex_new_called = FALSE;
+GMainLoop      *linc_loop = NULL;
+GMainContext   *linc_context = NULL;
 
 #ifdef LINC_SSL_SUPPORT
 SSL_METHOD *linc_ssl_method;
 SSL_CTX    *linc_ssl_ctx;
 #endif
 
+/**
+ * linc_set_threaded:
+ * @threaded: whether to do locking
+ * 
+ *   This routine turns threading on or off for the whole
+ * ORB, it should be called (TRUE) if threading is desired
+ * before any of the ORB initialization occurs.
+ **/
+void
+linc_set_threaded (gboolean threaded)
+{
+	if (linc_mutex_new_called)
+		g_error ("You need to set this before using the ORB");
+	linc_threaded = threaded;
+}
+
+/**
+ * linc_init:
+ * @init_threads: if we want threading enabled.
+ * 
+ * Initialize linc.
+ **/
 void
 linc_init (gboolean init_threads)
 {
-	if (init_threads && !g_thread_supported ())
+	if ((init_threads || linc_threaded) &&
+	    !g_thread_supported ())
 		g_thread_init (NULL);
 
-	g_type_init ();
+	if (!linc_threaded && init_threads)
+		linc_threaded = TRUE;
+
+	g_type_init_with_debug_flags (G_TYPE_DEBUG_OBJECTS);
 
 	/*
 	 * Linc's raison d'etre is for ORBit2 and Bonobo
@@ -74,6 +115,10 @@ struct _LincWatch {
 	guint linc_id;
 };
 
+#ifdef WATCH_DEBUG
+static GSList *watches = NULL;
+#endif
+
 LincWatch *
 linc_io_add_watch (GIOChannel    *channel,
 		   GIOCondition   condition,
@@ -89,46 +134,78 @@ linc_io_add_watch (GIOChannel    *channel,
 
 	/* Linc loop */
 	source = g_io_create_watch (channel, condition);
+	g_source_set_can_recurse (source, TRUE);
 	g_source_set_callback (source, (GSourceFunc)func, user_data, NULL);
 	w->linc_id = g_source_attach (source, linc_context);
 	g_source_unref (source);
 
 	/* Main loop */
 	source = g_io_create_watch (channel, condition);
+	g_source_set_can_recurse (source, TRUE);
 	g_source_set_callback (source, (GSourceFunc)func, user_data, NULL);
 	w->main_id = g_source_attach (source, NULL);
 	g_source_unref (source);
 
+#ifdef WATCH_DEBUG
+	watches = g_slist_prepend (watches, w);
+#endif
+	
 	return w;
 }
 
 void
-linc_io_remove_watch (LincWatch *watch)
+linc_io_remove_watch (LincWatch *w)
 {
-	if (watch) {
+	if (w) {
 		GSource *source;
 
+#ifdef WATCH_DEBUG
+		watches = g_slist_remove (watches, w);
+#endif
+
+		/* If these warnings fire, we're prolly returning
+		   TRUE from the source's handler somewhere in error */
+		   
 		source = g_main_context_find_source_by_id (
-			NULL, watch->main_id);
+			NULL, w->main_id);
 		if (source)
 			g_source_destroy (source);
 		else
 			g_warning ("Missing source on main context");
 
 		source = g_main_context_find_source_by_id (
-			linc_context, watch->linc_id);
+			linc_context, w->linc_id);
 		if (source)
 			g_source_destroy (source);
 		else
 			g_warning ("Missing source on linc context");
 
-		g_free (watch);
+		g_free (w);
 	}
 }
 
 void
 linc_main_iteration (gboolean block_for_reply)
 {
+#ifdef WATCH_DEBUG
+	GSList *l;
+
+	g_assert (watches != NULL);
+
+	for (l = watches; l; l = l->next) {
+		LincWatch *w = l->data;
+		GSource *source = g_main_context_find_source_by_id (
+			linc_context, w->linc_id);
+		GPollFD *fd;
+
+		g_warning ("source %p", source);
+
+		fd = source->poll_fds->data;
+		g_warning ("pollrec fd %d, (0x%x, 0x%x)",
+			   fd->fd, fd->events, fd->revents);
+	}
+#endif
+
 	g_main_context_iteration (
 		linc_context, block_for_reply);
 }
@@ -148,10 +225,13 @@ linc_main_loop_run (void)
 GMutex *
 linc_mutex_new (void)
 {
+	linc_mutex_new_called = TRUE;
+
 #ifdef G_THREADS_ENABLED
-	if (g_thread_supported ())
+	if (linc_threaded && g_thread_supported ())
 		return g_mutex_new ();
 #endif
 
 	return NULL;
 }
+
